@@ -18,15 +18,6 @@ import pandas as pd
 from shapely.geometry import box
 from tqdm import tqdm
 
-from osmsg.utils import (
-    create_charts,
-    create_profile_link,
-    download_osm_files,
-    get_file_path_from_url,
-    sum_tags,
-    verify_me_osm,
-)
-
 from .changefiles import (
     get_download_urls_changefiles,
     get_prev_hour,
@@ -40,6 +31,16 @@ from .changefiles import (
     strip_utc,
 )
 from .changesets import ChangesetToolKit
+from .utils import (
+    create_charts,
+    create_profile_link,
+    download_osm_files,
+    get_file_path_from_url,
+    sum_tags,
+    update_stats,
+    update_summary,
+    verify_me_osm,
+)
 
 users_temp = {}
 users = {}
@@ -134,6 +135,7 @@ def collect_changefile_stats(
         users[user].setdefault("countries", [])
         users[user].setdefault("hashtags", [])
         users[user].setdefault("editors", [])
+        summary_interval[timestamp].setdefault("editors", {})
 
         if changeset in processed_changesets:
             try:
@@ -147,11 +149,16 @@ def collect_changefile_stats(
                     for hstg in processed_changesets[changeset]["hashtags"]
                     if hstg not in users[user]["hashtags"]
                 ]
-                users[user]["editors"] += [
-                    editor
-                    for editor in processed_changesets[changeset]["editors"]
-                    if editor not in users[user]["editors"]
-                ]
+                for editor in processed_changesets[changeset]["editors"]:
+                    if editor not in users[user]["editors"]:
+                        users[user]["editors"].append(editor)
+                    # use regex to extract editor name
+                    pattern = r"([a-zA-Z\s]+)"
+                    editor_name = re.findall(pattern, editor)
+                    # convert to lowercase and print editor name
+                    editor = editor_name[0].lower()
+                    summary_interval[timestamp]["editors"].setdefault(editor, 0)
+                    summary_interval[timestamp]["editors"][editor] += 1
             except:
                 pass
 
@@ -374,7 +381,11 @@ def process_changesets(url):
     # print(f"Processing {url}")
     file_path = get_file_path_from_url(url, "changeset")
     handler = ChangesetHandler()
-    handler.apply_file(file_path[:-3])
+    try:
+        handler.apply_file(file_path[:-3])
+    except Exception as ex:
+        print(f"Error : {url}")
+        # raise ex
     # print(f"Finished {url}")
 
 
@@ -570,13 +581,6 @@ def parse_args():
         default=False,
     )
 
-    parser.add_argument(
-        "--include_date_in_name",
-        action="store_true",
-        help="You can append stats date and time in filename using this option",
-        default=False,
-    )
-
     parser.add_argument(
         "--format",
         nargs="+",
@@ -589,6 +593,12 @@ def parse_args():
         help="Location of metadata to pick start date from previous run's end_date , Generally used if you want to run bot on regular interval using cron/service",
     )
 
+    parser.add_argument(
+        "--update",
+        action="store_true",
+        default=False,
+        help="Update the old dataset produced by osmsg , Very Experimental : There should be your name stats.csv and summary.csv in place where command is run",
+    )
     args = parser.parse_args()
     return args
 
@@ -596,12 +606,24 @@ def parse_args():
 def main():
     args = parse_args()
     Initialize()
-    if args.start_date:
-        start_date = strip_utc(
-            dt.datetime.strptime(args.start_date, "%Y-%m-%d %H:%M:%S%z"), args.timezone
-        )
+    fname = args.name
+    full_path = os.path.abspath(
+        os.path.join(os.getcwd(), os.path.join(os.getcwd(), f"{fname}.csv"))
+    )
+    base_path = os.path.abspath(os.path.dirname(full_path))
+    print(base_path)
+    if not os.path.exists(base_path):
+        os.makedirs(base_path)
 
-    if not args.start_date:
+    if args.update:
+        if args.read_from_metadata:
+            print("Error : Option not allowed : read_from_metadata along with --update")
+            sys.exit()
+        if args.start_date:
+            print(
+                "Error : Start_date is not allowed during update it will read it from stats csv"
+            )
+            sys.exit()
         if (
             args.last_week
             or args.last_day
@@ -610,8 +632,31 @@ def main():
             or args.last_hour
             or args.days
         ):
-            pass
-        else:
+            print(
+                "Error : Can't pass last_* parameters along with update , update will pick start date from old csv and try to update up to now / end_date"
+            )
+            sys.exit()
+        old_csv_path = os.path.join(os.getcwd(), f"{fname}.csv")
+        if not os.path.exists(old_csv_path):
+            print(f"Error: Couldn't find old stats csv at :{old_csv_path}")
+        old_df = pd.read_csv(old_csv_path, encoding="utf8")
+        args.start_date = str(old_df.iloc[0]["end_date"])
+        old_stats_start_date = str(old_df.iloc[0]["start_date"])
+
+    if args.start_date:
+        start_date = strip_utc(
+            dt.datetime.strptime(args.start_date, "%Y-%m-%d %H:%M:%S%z"), args.timezone
+        )
+
+    if not args.start_date:
+        if not (
+            args.last_week
+            or args.last_day
+            or args.last_month
+            or args.last_year
+            or args.last_hour
+            or args.days
+        ):
             print(
                 "ERR: Supply start_date or extraction parameters such as last_day , last_hour"
             )
@@ -755,7 +800,7 @@ def main():
         sys.exit()
     if ((end_date - start_date).days < 1 or args.last_hour) and args.country:
         print(
-            "Warning : Use --changeset Option to include country info on stats , You can filter your country from output itself for stats lesser than a day, Use --country option for more than Day Statistics"
+            "Args country has day difference lesser than 1 day , Remove args country simply process a day data with --changeset option"
         )
         sys.exit()
     if (end_date - start_date).days > 1:
@@ -919,23 +964,6 @@ def main():
                                 )
                             )
                         )
-        if args.summary:
-            summary_df = pd.json_normalize(list(summary_interval.values()))
-            summary_df = summary_df.assign(
-                changes=summary_df["nodes.create"]
-                + summary_df["nodes.modify"]
-                + summary_df["nodes.delete"]
-                + summary_df["ways.create"]
-                + summary_df["ways.modify"]
-                + summary_df["ways.delete"]
-                + summary_df["relations.create"]
-                + summary_df["relations.modify"]
-                + summary_df["relations.delete"]
-            )
-            summary_df.insert(3, "map_changes", summary_df["changes"], True)
-            summary_df = summary_df.drop(columns=["changes"])
-
-            summary_df = summary_df.sort_values("timestamp", ascending=True)
 
         df = pd.json_normalize(list(users.values()))
 
@@ -975,13 +1003,51 @@ def main():
             df["hashtags"] = df["hashtags"].apply(lambda x: ",".join(map(str, x)))
             column_to_move = "hashtags"
             df = df.assign(**{column_to_move: df.pop(column_to_move)})
+            if args.summary:
+                for timestamp in summary_interval:
+                    summary_interval[timestamp]["editors"] = json.dumps(
+                        dict(
+                            sorted(
+                                summary_interval[timestamp]["editors"].items(),
+                                key=lambda item: item[1],
+                                reverse=True,
+                            )
+                        )
+                    )
+
+        if args.summary:
+            summary_df = pd.json_normalize(list(summary_interval.values()))
+            summary_df = summary_df.assign(
+                changes=summary_df["nodes.create"]
+                + summary_df["nodes.modify"]
+                + summary_df["nodes.delete"]
+                + summary_df["ways.create"]
+                + summary_df["ways.modify"]
+                + summary_df["ways.delete"]
+                + summary_df["relations.create"]
+                + summary_df["relations.modify"]
+                + summary_df["relations.delete"]
+            )
+            summary_df.insert(3, "map_changes", summary_df["changes"], True)
+            summary_df = summary_df.drop(columns=["changes"])
+
+            summary_df = summary_df.sort_values("timestamp", ascending=True)
+            if args.update:
+                old_summary_path = os.path.join(os.getcwd(), f"{fname}_summary.csv")
+                if not os.path.exists(old_summary_path):
+                    print(
+                        f"Error: Couldn't find old summary csv at :{old_summary_path}"
+                    )
+                old_summary = pd.read_csv(f"{fname}_summary.csv", encoding="utf8")
+                summary_df = update_summary(old_summary, summary_df)
 
         start_date = in_local_timezone(start_date_utc, args.timezone)
         end_date = in_local_timezone(end_date_utc, args.timezone)
-        fname = args.name
-        if args.include_date_in_name:
-            fname = f"""{args.name}_{start_date.strftime('%Y-%m-%d-%H-%M-%S%z')}_{end_date.strftime('%Y-%m-%d-%H-%M-%S%z')}"""
 
+        if args.update:
+            df = update_stats(old_df, df)
+
+        ### -------EXPORT BLOCK -------
         if "image" in args.format:  ### image used for twitter tweet
             # Convert the DataFrame to an image
             df_img = df.head(25)
@@ -1019,7 +1085,7 @@ def main():
 
             dfi.export(
                 result_df[cols_to_export],
-                "top_users.png",
+                f"{fname}_top_users.png",
                 max_cols=-1,
                 max_rows=-1,
             )
@@ -1031,10 +1097,14 @@ def main():
         if "csv" in args.format:
             # Add the start_date and end_date columns to the DataFrame
             csv_df = df
-            csv_df["start_date"] = start_date_utc
+            csv_df["start_date"] = (
+                old_stats_start_date if args.update else start_date_utc
+            )
             csv_df["end_date"] = end_date_utc
 
             # Create profile link column
+            csv_df.drop("profile", axis=1, inplace=True, errors="ignore")
+
             csv_df.insert(2, "profile", csv_df["name"].apply(create_profile_link))
 
             csv_df.to_csv(f"{fname}.csv", index=False)
@@ -1048,14 +1118,14 @@ def main():
                     f"User Contributions From {start_date} to {end_date} . Planet Source File : {args.url}\n "
                 )
                 file.write(text_output)
-
+        produced_charts_list = []
         if args.charts:
             if any("geofabrik" in url.lower() for url in args.url):
                 df.drop("countries", axis="columns")
-            create_charts(df)
+            produced_charts_list = create_charts(df, fname)
 
         if args.summary:
-            summary_df.to_csv(f"summary.csv", index=False)
+            summary_df.to_csv(f"{fname}_summary.csv", index=False)
             created_sum = (
                 df["nodes.create"] + df["ways.create"] + df["relations.create"]
             )
@@ -1070,9 +1140,9 @@ def main():
             summary_text = f"{humanize.intword(len(df))} Users made {humanize.intword(df['changesets'].sum())} changesets with {humanize.intword(df['map_changes'].sum())} map changes."
             thread_summary = f"{humanize.intword(created_sum.sum())} OSM Elements were Created, {humanize.intword(modified_sum.sum())} Modified & {humanize.intword(deleted_sum.sum())} Deleted."
 
-            with open(f"summary.md", "w", encoding="utf-8") as file:
+            with open(f"{fname}_summary.md", "w", encoding="utf-8") as file:
                 file.write(
-                    f"### Last Update : Stats from {start_date_utc} to {end_date_utc} (UTC Timezone)\n\n"
+                    f"### Last Update : Stats from {old_stats_start_date if args.update else start_date_utc} to {end_date_utc} (UTC Timezone)\n\n"
                 )
                 file.write(f"#### {summary_text}\n")
                 file.write(f"#### {thread_summary}\n")
@@ -1153,6 +1223,26 @@ def main():
                                 f"- {top_five.index[i]} : {top_five[i]} users\n"
                             )
                     file.write(f"{trending_countries}\n")
+                if args.charts:
+                    png_paths = []
+                    # loop through all files in the directory
+                    for ch in produced_charts_list:
+                        rel_path = os.path.relpath(
+                            os.path.join(os.getcwd(), ch), base_path
+                        )
+                        path_components = rel_path.split(os.sep)
+                        # remove any components that go "up" a directory
+                        while path_components[0] == "..":
+                            path_components.pop(0)
+                        # re-join the remaining components to get the modified relative path
+                        rel_path = "./" + os.sep.join(path_components)
+
+                        # add the relative path to the list
+                        png_paths.append(rel_path)
+                    file.write("\n Charts : \n")
+                    for chart in png_paths:
+                        file.write(f"![Alt text]({chart}) \n")
+
         # Loop through the arguments
         for i in range(len(sys.argv)):
             # If the argument is '--password'
